#ifndef PSOFIT_DATA_CPP
#define PSOFIT_DATA_CPP

#include "config.hpp"

/**
 * Impedance spectrum.
 * 
 * Format:
 * 
 * n
 * freq_1  real_1  imag_1
 * freq_2  real_1  imag_1
 * freq_2  real_1  imag_1
 * ...
 * freq_n  real_n  imag_n 
 */
struct Data : vector<Point> {
    void read(FILE* f) {
        int n;
        fscanf(f, "%d", &n);
        resize(n);
        
        for(int i = 0; i < n; ++i) {
            double freq, real, imag;
            fscanf(f, "%lf %lf %lf", &freq, &real, &imag);
            at(i) = Point(freq, compx(real, imag));
        }
    }
    
    void print(FILE* f) {
        int n = size();
        fprintf(f, "%d\n", n);
        
        for(int i = 0; i < n; ++i) {
            Point& p = at(i);
            fprintf(f, "%26.16lf %26.16lf %26.16lf\n", p.first, real(p.second), imag(p.second));
        }
    }
};

#endif
