/**
 * Usage:
 * 
 * ./psofit < input_data > fitted_params
 */

extern "C" {
    #include <cpso.h>
}

#include "data.cpp"
#include "params.cpp"
#include "utils.cpp"
#include "limits.cpp"

Data data;
Params params;

static double fitness(CpsoState* st, const double* v) {
    params.v = v;
    return -params.mse(data);
}

int main(int argc, char* argv[]) { 
    Limits lims;
    lims.read(stdin);
    data.read(stdin);
    
    CpsoState* st = cpso_new(N_PARAMS, 40);
    params.n = N_PARAMS;
    
    for(int i = 0; i < N_PARAMS; ++i) {
        Limit& l = lims[i];
        cpso_set_limits(st, i, l.first, l.second);
    }
    
    srand( random_seed() );
    cpso_set_eval(st, fitness, 0);
    cpso_init(st);
    cpso_run(st, N_ITERS, -ERR_STOP);
    
    double fit = cpso_get_best(st, -1, &params.v);
    
    params.print(stdout);
    fprintf(stdout, "%g\n", -fit);
    
    return 0;
}
