#ifndef PSOFIT_PARAMS_CPP
#define PSOFIT_PARAMS_CPP

#include "config.hpp"
#include "data.cpp"

/**
 * Function parameters.
 * 
 * 0: Rinf      1: R0       2: Tau
 * 3: Alpha     4: Beta
 * 5: Td_real   6: Td_imag
 */
struct Params {
    const double* v;
    int n;
    
    compx calc(double freq) {
        double omega = tp * freq;
        compx num = v[1] - v[0];
        
        compx aux = I * omega * v[2];
        if(v[3] != 1) aux = pow(aux, v[3]);
        
        compx den = 1.0 + aux;
        if(v[4] != 1) den = pow(den, v[4]);
        
        compx tfa;
        if(v[5] || v[6])
            tfa = exp(-omega * compx(v[5], v[6]));
        else
            tfa = 1;
        
        return (v[0] + num / den) * tfa;
    }
    
    double mse(const Data& d) {
        double acc = 0;
        int e = d.size();
        
        for(int i = 0; i < e; ++i) {
            compx y    = calc(d[i].first);
            double err = abs( y - d[i].second );
            acc += err * err;
        }

        return acc / e;
    }
    
    void print(FILE* f) {
        for(int i = 0; i < n; ++i) {
            fprintf(f, "%g\n", v[i]);
        }
    }
};

#endif
